%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% File: OspApplySearchPlan.m
% Author: Michael Morin
% Last modified on: May 29, 2011
% Description:
%  The OspApplySearchPlan function file.

%%
% Function: OspApplySearchPlan
% Description:
%  Apply an OSP search plan on a given initial probability of containment
%  vector.
% Input:
%  - poc_0: A row vector where poc_0(r) is the probability of containment
%    for region r at time 0.
%  - D: A markov motion model where D(s,r) is the probability of an object
%    move from region s to region r at any time step.
%  - pod: The probability of detection. The same probability is assumed
%    for all regions.
%  - Y: A T by N binary matrix where Y(t,r) = 1 if and only if one unit
%    of effort is allocated in region r at time step t.
%  - isOjbectMovingAtTime0: Equals 1 if and only if the search object is
%    allowed to move at time 0. Some OSP models doesn't allow such a move
%    while it is allowed in others.
% Output:
%  - e_POC: A T by N+1 matrix where e_POC(t,1:N) is the poc value of
%    all regions at time t and where e_POC(t,N+1) is the COS value at time
%    t.
% Note:
%  - T corresponds to the number of rows in Y.
%  - N corresponds to the number of column in Y.
%  - Other matrices size must agree.
function [e_POC] = OspApplySearchPlan(poc_0, D, pod, Y, ...
                                      isObjectMovingAtTime0)
    T = size(Y, 1);
    N = size(Y, 2);
    
    % Extended poc vector where e_poc[N + 1] = COS
    e_poc = [poc_0 0];
    
    % Motion model
    D = [D zeros(N,1) ; zeros(1,N) 1] 

    e_POC = [];
    for t = 1:1:T
        % Jk is used to compute the poc and the COS updates
        ek = Y(t,:)';
        Jk = [ (ek * ek') -ek; zeros(1,N) 0 ];

        if(t == 1 && isObjectMovingAtTime0 == 0)
            e_poc = e_poc * (eye(N + 1) - pod * Jk);
        else
            e_poc = e_poc * D * (eye(N + 1) - pod * Jk);
        end
        
        % Extended poc vector update
        e_POC = [ e_POC ; e_poc];
    end
end
