%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% File: OspGenerateMotionModel.m
% Author: Michael Morin
% Last modified on: May 29, 2011
% Description:
%  The OspGenerateMotionModel function file.

%%
% Function: OspGenerateMotionModel
% Description:
%  Generate a markovian motion model matrix.
% Input:
%  - D: D(s,s) is the probability that the object stays in the same region
%    at any time step.
%  - W: The number of columns.
%  - H: The number of rows.
% Output:
%  - D: A markov motion model where D(s,r) is the probability of an object
%    move from region s to region r at any time step.
function [D] = OspGenerateMotionModel(D_s_s, W, H)
    N = W * H;
    % Generate an adjacency matrix
    % See
    % http://stackoverflow.com/questions/3277541/construct-adjacency-matrix-in-matlab
    D_horizontal_connections = repmat([ones(W - 1,1); 0],H,1);
    D_horizontal_connections = D_horizontal_connections(1:end-1);
    D_vertical_connections = ones(W * (H - 1), 1);
    D_adj = diag(D_horizontal_connections,1) + diag(D_vertical_connections,W);
    D_adj = D_adj + D_adj'; % For symmetry
    D_adj = D_adj + eye(N); % A region is always accessible to itself
    
    % Generate
    D = [];
    i_col = 1;
    i_row = 1;
    for i = 1:N
        D_row = zeros(1,N);
        for j = 1:N
            if(i == j)
                D_row(j) = D_s_s;
            elseif(D_adj(i,j) == 1)
                D_row(j) = (1 - D_s_s) / (sum(D_adj(i,:)) - 1);
            end
            i_col = i_col + 1;
            if(i_col > 3) 
                i_col = 1;
                i_row = i_row + 1;
            end
        end
        D = [ D; D_row ];
    end                                      
end